import numpy as np


# minr, minc, maxr, maxc
# https://stackoverflow.com/questions/306316/determine-if-two-rectangles-overlap-each-other
def is_overlapping(box1, box2):
    #     if (RectA.minc < RectB.maxc && RectA.maxc > RectB.minc &&
    #      RectA.minr > RectB.maxr && RectA.maxr < RectB.minr )
    if box1[1] < box2[3] and box1[3] > box2[1] and box1[0] < box2[2] and box1[2] > box2[0]:
        return True
    return False


def is_almost_in_line(box1, box2):
    centroid_b1 = [int((box1[0] + box1[2]) / 2), int((box1[1] + box1[3]) / 2)]
    centroid_b2 = [int((box2[0] + box2[2]) / 2), int((box2[1] + box2[3]) / 2)]
    if centroid_b2[0] - centroid_b1[0] == 0:
        return True

    angle = (np.arctan(np.abs((centroid_b2[1] - centroid_b1[1]) / (centroid_b2[0] - centroid_b1[0]))) * 180) / np.pi
    if angle > 70:
        return True

    return False


def combine_boxes(box1, box2):
    minr = np.min([box1[0], box2[0]])
    minc = np.min([box1[1], box2[1]])
    maxr = np.max([box1[2], box2[2]])
    maxc = np.max([box1[3], box2[3]])
    return [minr, minc, maxr, maxc]


def group_the_bounding_boxes(bounding_boxes):
    grouped = []
    box_groups = []
    for iindex, box1 in enumerate(bounding_boxes):
        if iindex in grouped:
            continue

        bigger_box = box1
        group_size = 0
        for jindex, box2 in enumerate(bounding_boxes):
            if jindex in grouped:
                continue

            if iindex != jindex and is_overlapping(bigger_box, box2) and is_almost_in_line(bigger_box,
                                                                                           box2):
                bigger_box = combine_boxes(bigger_box, box2)

                grouped.append(iindex)  # gets added multiple times, still better than checking everytime.source
                grouped.append(jindex)
                group_size += 1

        if group_size > 1:
            box_groups.append(bigger_box)

    return box_groups



min_region_area = 15
min_aspect_ratio =  0.06
max_aspect_ratio = 3
max_eccentricity = 0.995
min_solidity = 0.3
min_region_extent = 0.2
max_region_extent = 0.9
min_euler_number = -4
min_stroke_width_metric = 0.4
expansion_amount = 0.02



# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test-text-detector
----------------------------------
Tests for `text-detector` module.
"""

import unittest

from utils.bbox import is_overlapping, is_almost_in_line, combine_boxes


class TestBboxUtils(unittest.TestCase):

    def test_bbox_is_overlapping(self):
        self.assertTrue(is_overlapping([1, 1, 3, 3], [2, 2, 4, 4]))
        self.assertTrue(is_overlapping([2, 2, 4, 4], [1, 1, 3, 3]))
        self.assertTrue(is_overlapping([2, 2, 4, 4], [2, 2, 4, 4]))
        self.assertFalse(is_overlapping([2, 2, 4, 4], [6, 2, 5, 4]))

    def test_bbox_is_almost_in_line(self):
        self.assertFalse(is_almost_in_line([1, 1, 3, 3], [2, 2, 4, 4]))
        self.assertFalse(is_almost_in_line([2, 2, 4, 4], [1, 1, 3, 3]))
        self.assertTrue(is_almost_in_line([2, 2, 4, 4], [2, 2, 4, 4]))
        self.assertFalse(is_almost_in_line([2, 2, 4, 4], [6, 2, 5, 4]))

    def test_bbox_combine_boxes(self):
        self.assertEqual([1, 1, 4, 4], combine_boxes([1, 1, 3, 3], [2, 2, 4, 4]))
        self.assertEqual([1, 1, 4, 4], combine_boxes([2, 2, 4, 4], [1, 1, 3, 3]))
        self.assertEqual([2, 2, 4, 4], combine_boxes([2, 2, 4, 4], [2, 2, 4, 4]))
        self.assertEqual([2, 2, 5, 4], combine_boxes([2, 2, 4, 4], [6, 2, 5, 4]))


if __name__ == '__main__':
    unittest.main()
